/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _flexCAM_h
#define _flexCAM_h

#include "flexSerial.h"
#include "flexUtils.h"

/** 
 * @mainpage Flex CAM Serial
 *
 * @section intro_sec Introduction 
 *
 * This application is for the PC side part of the Flex CAM Serial Demo. Currently, it runs under MS Windows with CygWin Posix environment, facilitating easy
 * adaptation of the code for a native Posix system.
 *
 * @section run Running 
 * The program must be executed in a command prompt window (Microsoft/Cygwin). 
 *
 * @subsection req_run Requirements
 *
 * The program uses cygwin1.dll (included in the distribution), that can either be shared through the windows PATH environment 
 * variable or be placed in the executable directory.
 *
 * @subsection usage_sec Usage 
 * <b>flex-cam-serial <video|snap>  <type> [port]</b>\n\n
 * \b video : Configure flex for video (continous image stream)\n
 * \b snap : Configure flex for snapshot (single image)\n
 * \b type : Select the type of image:\n
 * - gray2 - Gray scale 2 bit
 * - gray4 - Gray scale 4 bit
 * - gray8 - Gray scale 8 bit
 * - rgb16 - RGB 16 bit (R:5 - G:6 - B:5)
 * - rgb12 - RGB 12 bit (R:4 - G:4 - B:4)
 * - jpeg  - Jpeg compressed image
 *
 * \b port : the serial port to connect; default is com1 (windows)\n
 *
 * Examples of usage : 
 * - <c> flex-cam-serial video jpeg </c> 
 * - <c> flex-cam-serial video gray2 com1 </c>
 * - <c> flex-cam-serial snap  rgb12 </c>
 *
 * @subsection note_run Note 
 * The demo has been tested at 38400 bps baud rate.
 * Change the UART1 speed on the Flex side driver to try different baud rates.
 * The run-time errors are defined in the header file flexCAM.h.
 *
 * @section compilation Compilation 
 * The executable is build using the GNU toolchain under CygWin.
 * On the command prompt use <c> $ make </c> command to compile the project.
 * Use <c> $ make help </c> command to view build options
 * 
 * @subsection req_build Requirements
 *
 * Following CygWin packages:
 * - GNU toolchain (g++, make)
 * - libjpeg
 * - FLTK library, version 1.1.8 or higher
 * 
 * \b IMPORTANT
 *
 * Make following changes in the header file ‘jmorecfg.h’, usually located under ‘/urs/include’ sub-directory.
 *
 *\n
 * Replace
 * \code #ifndef HAVE_BOOLEAN \endcode
 * with 
 * \code #if (!defined(HAVE_BOOLEAN)) && (!defined(_RPCNDR_H)) \endcode
 * Replace
 * \code  #ifndef XMD_H \endcode
 * with 
 * \code #if (!defined(XMD_H) && !defined(_BASETSD_H)) \endcode
 *
 * @section lic License 
 * 
 * Copyright (C) 2007 - Nastasi Christian\n
 * This program is free software; you can redistribute it and/or\n
 * modify it under the terms of the GNU General Public License\n
 * as published by the Free Software Foundation; either version 2\n
 * of the License, or (at your option) any later version.\n
 * 
 * This program is distributed in the hope that it will be useful,\n
 * but WITHOUT ANY WARRANTY; without even the implied warranty of\n
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n
 * GNU General Public License for more details.\n
 * 
 * You should have received a copy of the GNU General Public License\n
 * along with this program; if not, write to the Free Software\n
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n
 * 
*/


/** 
* @file flexCAM.h
* @brief Flex-CAM Serial Driver
* @author Nastasi Christian
* @date 2007-12-17
*/

/** 
* @name Flex Driver Params
*
* Flex driver configuration register values
* @{ */
#define FLEX_CAM_GRAY2 0x1
#define FLEX_CAM_GRAY4 0x2 
#define FLEX_CAM_GRAY8 0x3 
#define FLEX_CAM_RGB12 0x5 
#define FLEX_CAM_RGB16 0x6 
#define FLEX_CAM_JPEG  0x7 

#define FLEX_CAM_SMALL 	0x1 
#define FLEX_CAM_MED 	0x3 
#define FLEX_CAM_LARGE 	0x5 
/**  @} */

/** 
* @name Error Codes
* @{ */
#define FLEX_ERR_SERIAL_CLOSED 		0	/*!< Serial Not Opened */
#define FLEX_ERR_SERIAL_OPEN 		1	/*!< Serial Opening Error */
#define FLEX_ERR_SERIAL_READ 		2	/*!< Serial Read Error */
#define FLEX_ERR_SERIAL_WRITE 		3	/*!< Serial Write Error */
#define FLEX_ERR_BAD_ACK 		4	/*!< Bad Ack from Flex */
#define FLEX_ERR_BAD_COLOR_TYPE		5	/*!< Bad Color Type */
#define FLEX_ERR_BAD_RES_TYPE		6 	/*!< Bad Resolution Type */
#define FLEX_ERR_NOT_INIT		7 	/*!< Not Initializated */
#define FLEX_ERR_NOT_CONFIG		8 	/*!< Not Configured */
#define FLEX_ERR_SERIAL_TIMEOUT		9	/*!< Serial I/O Timeout */
#define FLEX_ERR_JPEG_DECOMPRESS 	10	/*!< Jpeg decompression error */
/**  @} */

#ifdef __CYGWIN__ 
#define FLEX_DEFAULT_SERIAL_PORT "com1"		/*!< Default Serial Port Name */
#else
#define FLEX_DEFAULT_SERIAL_PORT "/dev/ttys0"
#endif

/** 
* @brief Flex-CAM Serial Driver
*/
class FlexCAM {

private:
	unsigned char color_type;	
	unsigned char resolution_type;
	unsigned long int baud_rate_pc;
	//unsigned char baud_rate_flex;
	bool video_stream;
	bool compression;
	
	Flex_Serial serial_port;

	unsigned int error_code;
	unsigned int width;
	unsigned int height;
	unsigned int channel;
	unsigned int data_size;

	unsigned char* buff1;
	unsigned char* buff2;

	bool connected;
	bool init;
	bool config;

public:
	
	/** 
	* @brief Initializes the private members
	*/
	FlexCAM();
	
	/** 
	* @brief Set the image stream properties
	* 
	* @param type The type of image
	* @param size The resolution of image
	* @param is_video Stream type is video or single snapshot
	* 
	* @return \c true if no error occurs, \c false otherwise
	*/
	bool set_image_type(unsigned char type, unsigned char size, bool is_video);

	/** 
	* @brief Set Flex-PC baud rate
	* 
	* @param b Baud Rate in bps
	*/
	inline void set_baud_rate_pc(unsigned long int b){ baud_rate_pc=b; };

	/** 
	* @brief Get the last error code
	* 
	* @return Last error code
	*/
	inline unsigned int get_error(){ return error_code; };
	/** 
	* @brief Get the image width
	* 
	* @return width
	*/
	inline unsigned int get_width(){ return width; };
	/** 
	* @brief Get the image height
	* 
	* @return height
	*/
	inline unsigned int get_height(){ return height; };
	/** 
	* @brief Get the number of channels 
	* 
	* The number of channels for the image can be 1(for gray images) or 3(for color images).
	* 
	* @return number of channel
	*/
	inline unsigned int get_channel(){ return channel; };
	
	/** 
	* @brief Open the serial port
	* 
	* If the \c serial_name is \c NULL than the default \ref FLEX_DEFAULT_SERIAL_PORT name is used.
	* 
	* @pre The \ref set_image_type method must be called first.
	* 
	* @param serial_name A valid serial port name 
	* 
	* @return \c true if no error occurs, \c false otherwise.
	*/
	bool connect(const char* serial_name);

	/** 
	* @brief Close the serial port
	*/
	void close();

	/** 
	* @brief Configure the Flex Driver
	* 
	* Set the Flex driver according to the configuration selected with the \ref set_image_type method.
	* 
	* @pre The \ref connect method must be called first.
	* 
	* @return \c true if no error occurs, \c false otherwise.
	*/
	bool configure(const char* serial_name);

	/** 
	* @brief Configure the Flex Driver
	* 
	* Query, grab and pre-process the next image from the Flex according the Flex driver configuration.\n
	* For the jpeg images the data size is unknown so it is performed byte to byte reading, looking for the jpeg starting and ending delimiters.\n
	* The pre-processing step converts images for future showing; color images in RGB24, gray images in Gray8. 
	* 
	* @pre The \ref configure method must be called first.
	* 
	* @param image The result pre-processed image data
	*
	* @return \c true if no error occurs, \c false otherwise.
	*/
	bool get_image(unsigned char* &image);
};


#endif


